/*
* Headers and functions for I2C
*
*/

#ifndef _I2C_FUNCTIONS_HEADER_
#define _I2C_FUNCTIONS_HEADER_

//## Header section ##########################################################################

#define I2CSTARTDELAY 5
#define I2CSTOPDELAY  5
#define I2CDATASETTLE 2
#define I2CCLOCKHIGH  10
#define I2CHALFCLOCK  5
#define I2CCLOCKLOW   10
#define I2CACKWAITMIN 6

//********************* I2C FUNCTIONS ************************
void i2csendack(void);			// Send acknowledge
void i2cdelay(uns8 delay);		// Delay
void i2cstart(void);			// Send START command
void i2cstop(void);				// Send STOP  command
void i2cclock(void);			// Clock generator

uns8 i2csendbyte(uns8 byte);	// Send 1 byte
uns8 i2cgetbyte(void);			// Receive 1 byte
uns8 i2cgetack(void);			// Read acknowledge
uns8 i2creadbit(void);			// Read 1 bit

//############################################################################################

#ifndef SCL
	#error Not defined SCL pin for I2C communication
#endif
#ifndef SDA
	#error Not defined SDA pin for I2C communication
#endif

#else

#pragma library 1	// Compile only used functions

//*************************************************************************
//                        i2cSendByte
//*************************************************************************
uns8 i2csendbyte(uns8 byte)
{
	uns8 count;			// Temporary byte for counter
	SDA=0;
	nop();
	SCL=0;

	i2cdelay(I2CCLOCKLOW);         // Minimum Clock Low Time

	for(count=8;count>0;count--)   // Send 8 bits of data
	{
		if( (byte & 0x80)== 0)     // Get the bit
		{
			SDA=0;                 // Ensure Port pin is low
			SDA_OUT;     	       // Lower pin if bit = 0
		}
		else
		{
			SDA_IN;				   // Release pin if bit = 1
		}
		byte=byte<<1;              // Shift next bit into position
		i2cclock();                // Pulse the clock
	}
	SDA_IN;              		   // Release data pin for ACK
	return(1);
}

//*************************************************************************
//                      uns8 i2cgetbyte(void)
//*************************************************************************
uns8 i2cgetbyte(void)
{
	uns8 count,byte;			   // Temporary byte for counter
	SDA=0;
	nop();
	SCL=0;
	byte=0;

	i2cdelay(I2CCLOCKLOW);         // Minimum Clock Low Time

	for(count=8;count>0;count--)   // Read 8 bits of data
	{
		byte=byte <<1;
		SDA_IN;            		   // Release pin so data can be received
		if(i2creadbit()) byte +=1;
	}
	return(byte);
}

//*************************************************************************
//                           i2cstart
//*************************************************************************
void i2cstart(void)
{
	SDA_IN;						   // Pins are in high impedance
	SCL_IN;
	SCL=0;
	nop();
	SDA=0;
	i2cdelay(I2CSTARTDELAY);

	SDA_OUT;			           // Generate the start condition
	SDA=0;
	i2cdelay(I2CSTARTDELAY);
	SCL_OUT;
	i2cdelay(I2CCLOCKLOW);         // Minimum Clock Low Time
}

//*************************************************************************
//                           i2cStop
//*************************************************************************
void i2cstop(void)
{
	SDA_OUT;
	SDA=0;
	i2cdelay(I2CSTOPDELAY);
	SDA=1;
	SCL_OUT;
	SCL=1;
}

//*************************************************************************
//                          i2cclock
//*************************************************************************
void i2cclock(void)
{
	i2cdelay(I2CDATASETTLE);       // Minimum Clock Low Time
	SCL_IN;              		   // Release clock
	i2cdelay(I2CCLOCKHIGH);        // Minimum Clock High Time
	SCL_OUT;               	       // Lower the clock
	SCL=0;
	i2cdelay(I2CCLOCKLOW);         // Minimum Clock Low Time
}

//*************************************************************************
//                          i2creadbit
//*************************************************************************
uns8 i2creadbit(void)
{
	uns8 Data=0;

	SCL=0;

	i2cdelay(I2CDATASETTLE);	   // Minimum Clock Low Time
	SCL_IN;						   // Release clock
	i2cdelay(I2CHALFCLOCK);		   // 1/2 Minimum Clock High Time
	if (SDA) Data=1;			   // READ in the data bit
	i2cdelay(I2CHALFCLOCK);		   // 1/2 Minimum Clock High Time
	SCL_OUT;					   // Lower the clock
	i2cdelay(I2CCLOCKLOW);		   // Minimum Clock Low Time
	return(Data);
}

//*************************************************************************
//                           i2cgetack
//*************************************************************************
uns8 i2cgetack(void)
{
	SDA=0;
	nop();
	SCL=0;

	SCL_OUT;					   // Ensure clock is low
	SDA_IN;						   // Release the Data pin so slave can ACK
	SCL_IN;		 				   // raise the clock pin
	i2cdelay(I2CHALFCLOCK);		   // wait for 1/2 the clock pulse
	if (SDA) return(0);			   // sample the ACK signal
	i2cdelay(I2CHALFCLOCK);		   // Else wait for rest of clock
	SCL_OUT;					   // Finish the clock pulse
	i2cdelay(I2CCLOCKLOW);		   // Minimum Clock Low Time
	i2cdelay(I2CCLOCKLOW);		   // Minimum Clock Low Time
	return(1);
}

//*************************************************************************
//                   void i2csendack(void)
//*************************************************************************
void i2csendack(void)
{
	SDA=0;

	SDA_OUT;					   // Send ACK
	i2cdelay(I2CDATASETTLE);	   // Ttime to settle
	i2cclock();					   // Pulse the clock
	SDA_IN;						   // Release ACK
	i2cdelay(I2CDATASETTLE);	   // Gap for next byte
}

//*************************************************************************
//                           i2cdelay
//*************************************************************************
void i2cdelay(uns8 delay)
{
	while (delay != 0)
	{
	#asm
		nop
		nop
		nop
		nop
	#endasm
	delay--;
	}
}
// ############################################################################################
#pragma library 0	// Compile all

#endif
// ############################################################################################
